local Grid2LuaTextsOptions = {}
Grid2Options.plugins["Grid2LuaTextsOptions"] = Grid2LuaTextsOptions

local L = LibStub("AceLocale-3.0"):GetLocale("Grid2Options")
local DBL = LibStub:GetLibrary("LibDBLayers-1.0")

local luaTextValues = {}
function Grid2Options:GetLuaTextValues(base)
	local values = luaTextValues[base]
	if (not values) then
		values = {}
		local codes = Grid2.luaTexts[base]
		for name, entry in pairs(codes) do
			values[name] = name
		end
		luaTextValues[base] = values
	end
	return values
end
--/dump Grid2Options:GetLuaTextValues("Health")

local default = L["Grid2"]
function Grid2Options:MakeStatusLuaTextOptions(status, options, optionParams)
	options = options or {}
	options.format = {
	    type = 'select',
		order = 110,
		name = L["Format"],
		desc = L["Format to display the text as."],
	    values = function ()
			local base = optionParams.base
			return Grid2Options:GetLuaTextValues(base)
		end,
		get = function ()
			local entry = status.dbx.entry or default
			return entry
		end,
		set = function (info, v)
			if (v == default) then
				v = nil
			end
			status.dbx.entry = v
			DBL:GetOptionsDbx(Grid2.dblData, "statuses", status.name).entry = v
			status.dbx.base = optionParams.base
			DBL:GetOptionsDbx(Grid2.dblData, "statuses", status.name).base = optionParams.base
			if status.UpdateProfileData then
				status:UpdateProfileData()
			end
			Grid2:MakeTextHandler(status)
			for unit, guid in Grid2:IterateRosterUnits() do
				status:UpdateIndicators(unit)
			end
		end,
	}
	
	return options
end

function Grid2Options:MakeWrappedLuaTextOptions(typeKey, params)
	local funcMakeOptions, optionParams = Grid2Options:GetOptionHandler(typeKey)
	optionParams = optionParams or {}
	if (params) then
		DBL:CopyDB(params, optionParams)
	end
	return function(self, status, options, optionParams)
		options = funcMakeOptions(self, status, options, optionParams)
		options = Grid2Options:MakeStatusLuaTextOptions(status, options, optionParams)
		return options
	end, optionParams
end


function Grid2LuaTextsOptions.UpgradeDefaults(dblData)
	local versionsSrc = dblData.versionsSrc

	if (versionsSrc.Grid2LuaTextsOptions < 1) then
		versionsSrc.Grid2LuaTextsOptions = 1
	end
end

local prev_MakeStatusHandlers = Grid2Options.MakeStatusHandlers
function Grid2Options:MakeStatusHandlers(dblData, reset, ...)
	prev_MakeStatusHandlers(self, dblData, reset, ...)

	-- Overide the old handlers
	self:AddOptionHandler("health-current", Grid2Options:MakeWrappedLuaTextOptions("health-current", {
			base = L["Health"],
	}))
	self:AddOptionHandler("mana", Grid2Options:MakeWrappedLuaTextOptions("mana", {
			base = L["Power"],
	}))
end

